"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReportService = void 0;
const LoggerFactory_1 = require("../logger/LoggerFactory");
const LogType_1 = require("../logger/LogType");
const BootstrapUtils_1 = require("./BootstrapUtils");
const path_1 = require("path");
const fs_1 = require("fs");
const _ = require("lodash");
const logger = LoggerFactory_1.default.getLogger(LogType_1.LogType.System);
class ReportService {
    constructor(root, params) {
        this.root = root;
        this.params = params;
    }
    createReportFromFile(resourceContent, descriptions) {
        const sections = [];
        const lines = resourceContent
            .trim()
            .split(/\s*[\r\n]+\s*/g)
            .map((l) => l.trim())
            .filter((l) => l && l.indexOf('#') != 0);
        lines.forEach((l) => {
            const isHeader = /\[(.*?)\]/.exec(l);
            if (isHeader && isHeader.length && isHeader[1]) {
                sections.push({
                    header: isHeader[1],
                    lines: [],
                });
            }
            else {
                const isProperty = /(.*)=(.*)/.exec(l);
                if (isProperty && isProperty.length && isProperty[1]) {
                    const propertyName = isProperty[1].trim();
                    const descriptor = descriptions[propertyName];
                    const hidden = (descriptor && descriptor.hidden) || false;
                    const value = isProperty[2].trim();
                    sections[sections.length - 1].lines.push({
                        property: propertyName,
                        value: hidden ? value.replace(/./g, '*') : value,
                        hidden: hidden,
                        description: (descriptor && descriptor.description) || '',
                        type: (descriptor && descriptor.type) || '',
                    });
                }
            }
        });
        return sections;
    }
    async createReportsPerNode(presetData) {
        const workingDir = process.cwd();
        const target = path_1.join(workingDir, this.params.target);
        const descriptions = await BootstrapUtils_1.BootstrapUtils.loadYaml(path_1.join(this.root, 'presets', 'descriptions.yml'));
        const promises = (presetData.nodes || []).map(async (n) => {
            const resourcesFolder = path_1.join(target, 'config', n.name, 'resources');
            const files = await fs_1.promises.readdir(resourcesFolder);
            const reportFiles = files
                .filter((fileName) => fileName.indexOf('.properties') > -1)
                .map((fileName) => {
                const resourceContent = fs_1.readFileSync(path_1.join(resourcesFolder, fileName), 'utf-8');
                const sections = this.createReportFromFile(resourceContent, descriptions);
                const reportFile = {
                    fileName: fileName,
                    sections,
                };
                return reportFile;
            });
            const reportNode = {
                name: n.name,
                files: reportFiles,
            };
            return reportNode;
        });
        return Promise.all(promises);
    }
    /**
     *  It generates a .rst config report file per node.
     * @param passedPresetData the preset data,
     */
    async run(passedPresetData) {
        const presetData = passedPresetData !== null && passedPresetData !== void 0 ? passedPresetData : BootstrapUtils_1.BootstrapUtils.loadExistingPresetData(this.params.target);
        const reportFolder = path_1.join(this.params.target, 'report');
        BootstrapUtils_1.BootstrapUtils.deleteFolder(reportFolder);
        const reportNodes = await this.createReportsPerNode(presetData);
        const missingProperties = _.flatMap(reportNodes, (n) => _.flatMap(n.files, (f) => _.flatMap(f.sections, (s) => s.lines.filter((s) => !s.type && !s.description && !s.property.startsWith('starting-at-height')).map((s) => s.property))));
        const missingDescriptions = _.uniq(missingProperties);
        if (missingDescriptions.length) {
            const missingDescriptionsObject = _.mapValues(_.keyBy(missingProperties, (k) => k), () => ({
                type: '',
                description: '',
            }));
            logger.debug('Missing yaml properties: ' + BootstrapUtils_1.BootstrapUtils.toYaml(missingDescriptionsObject));
        }
        // const missingDescriptions = reportNodes.map(node -> node.files)
        await BootstrapUtils_1.BootstrapUtils.mkdir(reportFolder);
        const promises = _.flatMap(reportNodes, (n) => {
            return [this.toRstReport(reportFolder, n), this.toCsvReport(reportFolder, n)];
        });
        return Promise.all(promises);
    }
    async toRstReport(reportFolder, n) {
        const reportFile = path_1.join(reportFolder, `${n.name}-config.rst`);
        const reportContent = n.files
            .map((fileReport) => {
            const hasDescriptionSection = fileReport.sections.find((s) => s.lines.find((l) => l.description || l.type));
            const header = hasDescriptionSection ? '"Property", "Value", "Type", "Description"' : '"Property", "Value"';
            const csvBody = fileReport.sections
                .map((s) => {
                const hasDescriptionSection = s.lines.find((l) => l.description || l.type);
                return ((hasDescriptionSection ? `**${s.header}**; ; ;\n` : `**${s.header}**;\n`) +
                    s.lines
                        .map((l) => {
                        if (hasDescriptionSection)
                            return `${l.property}; ${l.value}; ${l.type}; ${l.description}`.trim() + '\n';
                        else {
                            return `${l.property}; ${l.value}`.trim() + '\n';
                        }
                    })
                        .join(''));
            })
                .join('');
            return `
${fileReport.fileName}
${fileReport.fileName.replace(/./g, '=')}
.. csv-table::
    :header: ${header}
    :delim: ;

${csvBody.trim().replace(/^/gm, '    ')}`;
        })
            .join('\n');
        await BootstrapUtils_1.BootstrapUtils.writeTextFile(reportFile, reportContent);
        logger.info(`Report file ${reportFile} created`);
        return reportFile;
    }
    async toCsvReport(reportFolder, n) {
        const reportFile = path_1.join(reportFolder, `${n.name}-config.csv`);
        const reportContent = n.files
            .map((fileReport) => {
            const csvBody = fileReport.sections
                .map((s) => {
                const hasDescriptionSection = s.lines.find((l) => l.description || l.type);
                return (`${s.header}\n` +
                    s.lines
                        .map((l) => {
                        if (hasDescriptionSection)
                            return `${l.property}; ${l.value}; ${l.type}; ${l.description}`.trim() + '\n';
                        else {
                            return `${l.property}; ${l.value}`.trim() + '\n';
                        }
                    })
                        .join(''));
            })
                .join('\n');
            return `${fileReport.fileName}
${csvBody.trim()}`;
        })
            .join('\n\n\n');
        await BootstrapUtils_1.BootstrapUtils.writeTextFile(reportFile, reportContent);
        logger.info(`Report file ${reportFile} created`);
        return reportFile;
    }
}
exports.ReportService = ReportService;
ReportService.defaultParams = {
    target: 'target',
};
